#ifndef USERINTERFACE_H_
#define USERINTERFACE_H_

#include <msp/gltk/resources.h>
#include <msp/gltk/root.h>
#include "libr2c2/driver.h"
#include "libr2c2/train.h"
#include "libr2c2/zone.h"
#include "mainwindow.h"

class DynamicDialog;

class UserInterface
{
public:
	class Loader: public Msp::DataFile::ObjectLoader<UserInterface>
	{
	public:
		Loader(UserInterface &);

	private:
		template<typename T>
		void dialog(T &);

		template<typename T>
		void basic_dialog();

		void traindialog(unsigned);
	};

private:
	Engineer &engineer;
	Msp::GLtk::Resources resources;
	Msp::GLtk::Root root;
	Msp::GLtk::Arrangement *root_arrangement;

	MainWindow main_wnd;
	std::set<DynamicDialog *> dyn_dialogs;
	bool import_active;

	Msp::Time::TimeStamp status_timeout;

public:
	UserInterface(Engineer &, Msp::Graphics::Window &, Msp::Input::Keyboard &, Msp::Input::Mouse &);
	~UserInterface();

	const Msp::GLtk::Resources &get_resources() const { return resources; }

	void set_status(const std::string &);
	void add_dynamic_dialog(DynamicDialog &);
	void remove_dynamic_dialog(DynamicDialog &);

	void show_train(R2C2::Train &);
	void show_zone(R2C2::Zone &);
private:
	void locomotive_detected(const R2C2::Driver::DetectedLocomotive &);
	void import_finished(int);

public:
	void tick();
	void render() const;

	void save_state(const std::string &) const;
};

#endif
