#ifndef LIBR2C2_TRAINROUTEMETRIC_H_
#define LIBR2C2_TRAINROUTEMETRIC_H_

#include <map>
#include <vector>
#include "trackchain.h"

namespace R2C2 {

class Layout;
class Track;

// Metrics store iterators facing away from the goal
class TrainRouteMetric
{
private:
	typedef std::pair<const Track *, unsigned> Key;

	struct Goal
	{
		TrackIter track;
		float base_distance;

		Goal();
		Goal(const TrackIter &);
	};

	struct Data
	{
		float distance;
		const Goal *goal;

		Data();
		Data(float, const Goal *);
	};

	std::vector<Goal> goals;
	std::map<Key, Data> tracks;

public:
	TrainRouteMetric() { }
	TrainRouteMetric(const TrackChain &, TrackChain::Direction = TrackChain::UNSPECIFIED);

	void chain_to(const TrainRouteMetric &);
	float get_distance_from(const Track &) const;
	float get_distance_from(const Track &, unsigned) const;
	float get_travel_multiplier(const Track &, unsigned) const;
};

} // namespace R2C2

#endif
